import { Event } from "../../../../base/common/event.js";
import { IMarkdownString } from "../../../../base/common/htmlContent.js";
import { Disposable } from "../../../../base/common/lifecycle.js";
import { ThemeIcon } from "../../../../base/common/themables.js";
import { URI, UriComponents, UriDto } from "../../../../base/common/uri.js";
import { IOffsetRange } from "../../../../editor/common/core/offsetRange.js";
import { IRange } from "../../../../editor/common/core/range.js";
import { Location, SymbolKind, TextEdit } from "../../../../editor/common/languages.js";
import { ILogService } from "../../../../platform/log/common/log.service.js";
import { ChatAgentLocation, IChatAgentCommand, IChatAgentData, IChatAgentResult, IChatWelcomeMessageContent } from "./chatAgents.js";
import { IChatAgentService } from "./chatAgents.service.js";
import { IParsedChatRequest } from "./chatParserTypes.js";
import { ChatAgentVoteDirection, ChatAgentVoteDownReason, IChatAgentMarkdownContentWithVulnerability, IChatCodeCitation, IChatCommandButton, IChatConfirmation, IChatContentInlineReference, IChatContentReference, IChatFollowup, IChatLocationData, IChatMarkdownContent, IChatProgress, IChatProgressMessage, IChatResponseCodeblockUriPart, IChatResponseProgressFileTreeData, IChatTask, IChatTextEdit, IChatToolInvocation, IChatToolInvocationSerialized, IChatTreeData, IChatUsedContext, IChatWarningMessage } from "./chatService.js";
import { IChatRequestVariableValue } from "./chatVariables.js";
export interface IBaseChatRequestVariableEntry {
    id: string;
    fullName?: string;
    icon?: ThemeIcon;
    name: string;
    isMarkedReadonly?: boolean;
    modelDescription?: string;
    range?: IOffsetRange;
    value: IChatRequestVariableValue;
    references?: IChatContentReference[];
    mimeType?: string;
    kind?: never;
    isDynamic?: boolean;
    isFile?: boolean;
    isDirectory?: boolean;
    isTool?: boolean;
    isImage?: boolean;
}
export interface IChatRequestImplicitVariableEntry extends Omit<IBaseChatRequestVariableEntry, "kind"> {
    readonly kind: "implicit";
    readonly isDynamic: true;
    readonly isFile: true;
    readonly value: URI | Location | undefined;
    readonly isSelection: boolean;
    enabled: boolean;
}
export interface IChatRequestPasteVariableEntry extends Omit<IBaseChatRequestVariableEntry, "kind"> {
    readonly kind: "paste";
    code: string;
    language: string;
    pastedLines: string;
    fileName: string;
    copiedFrom: {
        readonly uri: URI;
        readonly range: IRange;
    } | undefined;
}
export interface ISymbolVariableEntry extends Omit<IBaseChatRequestVariableEntry, "kind"> {
    readonly kind: "symbol";
    readonly isDynamic: true;
    readonly value: Location;
    readonly symbolKind: SymbolKind;
}
export interface ICommandResultVariableEntry extends Omit<IBaseChatRequestVariableEntry, "kind"> {
    readonly kind: "command";
    readonly isDynamic: true;
}
export interface ILinkVariableEntry extends Omit<IBaseChatRequestVariableEntry, "kind"> {
    readonly kind: "link";
    readonly isDynamic: true;
    readonly value: URI;
}
export type IChatRequestVariableEntry = IChatRequestImplicitVariableEntry | IChatRequestPasteVariableEntry | ISymbolVariableEntry | ICommandResultVariableEntry | ILinkVariableEntry | IBaseChatRequestVariableEntry;
export declare function isImplicitVariableEntry(obj: IChatRequestVariableEntry): obj is IChatRequestImplicitVariableEntry;
export declare function isPasteVariableEntry(obj: IChatRequestVariableEntry): obj is IChatRequestPasteVariableEntry;
export declare function isLinkVariableEntry(obj: IChatRequestVariableEntry): obj is ILinkVariableEntry;
export declare function isChatRequestVariableEntry(obj: unknown): obj is IChatRequestVariableEntry;
export interface IChatRequestVariableData {
    variables: IChatRequestVariableEntry[];
}
export interface IChatRequestModel {
    readonly id: string;
    readonly timestamp: number;
    readonly username: string;
    readonly avatarIconUri?: URI;
    readonly session: IChatModel;
    readonly message: IParsedChatRequest;
    readonly attempt: number;
    readonly variableData: IChatRequestVariableData;
    readonly confirmation?: string;
    readonly locationData?: IChatLocationData;
    readonly attachedContext?: IChatRequestVariableEntry[];
    readonly workingSet?: URI[];
    readonly isCompleteAddedRequest: boolean;
    readonly paused: boolean;
    readonly response?: IChatResponseModel;
    shouldBeRemovedOnSend: boolean;
}
export interface IChatTextEditGroupState {
    sha1: string;
    applied: number;
}
export interface IChatTextEditGroup {
    uri: URI;
    edits: TextEdit[][];
    state?: IChatTextEditGroupState;
    kind: "textEditGroup";
    done: boolean | undefined;
}
export type IChatProgressHistoryResponseContent = IChatMarkdownContent | IChatAgentMarkdownContentWithVulnerability | IChatResponseCodeblockUriPart | IChatTreeData | IChatContentInlineReference | IChatProgressMessage | IChatCommandButton | IChatWarningMessage | IChatTask | IChatTextEditGroup | IChatConfirmation;
export type IChatProgressResponseContent = IChatProgressHistoryResponseContent | IChatToolInvocation | IChatToolInvocationSerialized;
export declare function toChatHistoryContent(content: ReadonlyArray<IChatProgressResponseContent>): IChatProgressHistoryResponseContent[];
export type IChatProgressRenderableResponseContent = Exclude<IChatProgressResponseContent, IChatContentInlineReference | IChatAgentMarkdownContentWithVulnerability | IChatResponseCodeblockUriPart>;
export interface IResponse {
    readonly value: ReadonlyArray<IChatProgressResponseContent>;
    getMarkdown(): string;
    toString(): string;
}
export interface IChatResponseModel {
    readonly onDidChange: Event<void>;
    readonly id: string;
    readonly requestId: string;
    readonly username: string;
    readonly avatarIcon?: ThemeIcon | URI;
    readonly session: IChatModel;
    readonly agent?: IChatAgentData;
    readonly usedContext: IChatUsedContext | undefined;
    readonly contentReferences: ReadonlyArray<IChatContentReference>;
    readonly codeCitations: ReadonlyArray<IChatCodeCitation>;
    readonly progressMessages: ReadonlyArray<IChatProgressMessage>;
    readonly slashCommand?: IChatAgentCommand;
    readonly agentOrSlashCommandDetected: boolean;
    readonly response: IResponse;
    readonly isComplete: boolean;
    readonly isCanceled: boolean;
    readonly isPendingConfirmation: boolean;
    readonly shouldBeRemovedOnSend: boolean;
    readonly isCompleteAddedRequest: boolean;
    readonly isStale: boolean;
    readonly vote: ChatAgentVoteDirection | undefined;
    readonly voteDownReason: ChatAgentVoteDownReason | undefined;
    readonly followups?: IChatFollowup[] | undefined;
    readonly result?: IChatAgentResult;
    setVote(vote: ChatAgentVoteDirection): void;
    setVoteDownReason(reason: ChatAgentVoteDownReason | undefined): void;
    setEditApplied(edit: IChatTextEditGroup, editCount: number): boolean;
}
export declare class ChatRequestModel implements IChatRequestModel {
    private _session;
    readonly message: IParsedChatRequest;
    private _variableData;
    readonly timestamp: number;
    private _attempt;
    private _confirmation?;
    private _locationData?;
    private _attachedContext?;
    private _workingSet?;
    readonly isCompleteAddedRequest: boolean;
    response: ChatResponseModel | undefined;
    readonly id: string;
    get session(): ChatModel;
    shouldBeRemovedOnSend: boolean;
    get username(): string;
    get avatarIconUri(): URI | undefined;
    get attempt(): number;
    get variableData(): IChatRequestVariableData;
    set variableData(v: IChatRequestVariableData);
    get confirmation(): string | undefined;
    get locationData(): IChatLocationData | undefined;
    get attachedContext(): IChatRequestVariableEntry[] | undefined;
    get workingSet(): URI[] | undefined;
    private _paused;
    get paused(): boolean;
    set paused(paused: boolean);
    constructor(_session: ChatModel, message: IParsedChatRequest, _variableData: IChatRequestVariableData, timestamp: number, _attempt?: number, _confirmation?: string | undefined, _locationData?: IChatLocationData | undefined, _attachedContext?: IChatRequestVariableEntry[] | undefined, _workingSet?: URI[] | undefined, isCompleteAddedRequest?: boolean, restoredId?: string);
    adoptTo(session: ChatModel): void;
}
export declare class Response extends Disposable implements IResponse {
    private _onDidChangeValue;
    get onDidChangeValue(): Event<void>;
    private _responseParts;
    private _responseRepr;
    private _markdownContent;
    private _citations;
    get value(): IChatProgressResponseContent[];
    constructor(value: IMarkdownString | ReadonlyArray<IMarkdownString | IChatResponseProgressFileTreeData | IChatContentInlineReference | IChatAgentMarkdownContentWithVulnerability | IChatResponseCodeblockUriPart>);
    toString(): string;
    getMarkdown(): string;
    clear(): void;
    updateContent(progress: IChatProgressResponseContent | IChatTextEdit | IChatTask, quiet?: boolean): void;
    addCitation(citation: IChatCodeCitation): void;
    private _updateRepr;
    private partsToRepr;
    private inlineRefToRepr;
    private uriToRepr;
}
export declare class ChatResponseModel extends Disposable implements IChatResponseModel {
    private _session;
    private _agent;
    private _slashCommand;
    readonly requestId: string;
    private _isComplete;
    private _isCanceled;
    private _vote?;
    private _voteDownReason?;
    private _result?;
    readonly isCompleteAddedRequest: boolean;
    private _shouldBeRemovedOnSend;
    private readonly _onDidChange;
    readonly onDidChange: Event<void>;
    readonly id: string;
    get session(): ChatModel;
    get shouldBeRemovedOnSend(): boolean;
    get isComplete(): boolean;
    set shouldBeRemovedOnSend(hidden: boolean);
    get isCanceled(): boolean;
    get vote(): ChatAgentVoteDirection | undefined;
    get voteDownReason(): ChatAgentVoteDownReason | undefined;
    get followups(): IChatFollowup[] | undefined;
    private _response;
    get response(): IResponse;
    get result(): IChatAgentResult | undefined;
    get username(): string;
    get avatarIcon(): ThemeIcon | URI | undefined;
    private _followups?;
    get agent(): IChatAgentData | undefined;
    get slashCommand(): IChatAgentCommand | undefined;
    private _agentOrSlashCommandDetected;
    get agentOrSlashCommandDetected(): boolean;
    private _usedContext;
    get usedContext(): IChatUsedContext | undefined;
    private readonly _contentReferences;
    get contentReferences(): ReadonlyArray<IChatContentReference>;
    private readonly _codeCitations;
    get codeCitations(): ReadonlyArray<IChatCodeCitation>;
    private readonly _progressMessages;
    get progressMessages(): ReadonlyArray<IChatProgressMessage>;
    private _isStale;
    get isStale(): boolean;
    get isPendingConfirmation(): boolean;
    constructor(_response: IMarkdownString | ReadonlyArray<IMarkdownString | IChatResponseProgressFileTreeData | IChatContentInlineReference | IChatAgentMarkdownContentWithVulnerability | IChatResponseCodeblockUriPart>, _session: ChatModel, _agent: IChatAgentData | undefined, _slashCommand: IChatAgentCommand | undefined, requestId: string, _isComplete?: boolean, _isCanceled?: boolean, _vote?: ChatAgentVoteDirection | undefined, _voteDownReason?: ChatAgentVoteDownReason | undefined, _result?: IChatAgentResult | undefined, followups?: ReadonlyArray<IChatFollowup>, isCompleteAddedRequest?: boolean, _shouldBeRemovedOnSend?: boolean, restoredId?: string);
    updateContent(responsePart: IChatProgressResponseContent | IChatTextEdit, quiet?: boolean): void;
    applyReference(progress: IChatUsedContext | IChatContentReference): void;
    applyCodeCitation(progress: IChatCodeCitation): void;
    setAgent(agent: IChatAgentData, slashCommand?: IChatAgentCommand): void;
    setResult(result: IChatAgentResult): void;
    complete(): void;
    cancel(): void;
    setFollowups(followups: IChatFollowup[] | undefined): void;
    setVote(vote: ChatAgentVoteDirection): void;
    setVoteDownReason(reason: ChatAgentVoteDownReason | undefined): void;
    setEditApplied(edit: IChatTextEditGroup, editCount: number): boolean;
    adoptTo(session: ChatModel): void;
}
export declare enum ChatPauseState {
    NotPausable = 0,
    Paused = 1,
    Unpaused = 2
}
export interface IChatModel {
    readonly onDidDispose: Event<void>;
    readonly onDidChange: Event<IChatChangeEvent>;
    readonly sessionId: string;
    readonly initState: ChatModelInitState;
    readonly initialLocation: ChatAgentLocation;
    readonly title: string;
    readonly welcomeMessage: IChatWelcomeMessageContent | undefined;
    readonly sampleQuestions: IChatFollowup[] | undefined;
    readonly requestInProgress: boolean;
    readonly requestPausibility: ChatPauseState;
    readonly inputPlaceholder?: string;
    toggleLastRequestPaused(paused?: boolean): void;
    disableRequests(requestIds: ReadonlyArray<string>): void;
    getRequests(): IChatRequestModel[];
    toExport(): IExportableChatData;
    toJSON(): ISerializableChatData;
}
export interface ISerializableChatsData {
    [sessionId: string]: ISerializableChatData;
}
export type ISerializableChatAgentData = UriDto<IChatAgentData>;
export interface ISerializableChatRequestData {
    requestId: string;
    message: string | IParsedChatRequest;
    variableData: IChatRequestVariableData;
    response: ReadonlyArray<IMarkdownString | IChatResponseProgressFileTreeData | IChatContentInlineReference | IChatAgentMarkdownContentWithVulnerability> | undefined;
    isHidden: boolean;
    responseId?: string;
    agent?: ISerializableChatAgentData;
    workingSet?: UriComponents[];
    slashCommand?: IChatAgentCommand;
    result?: IChatAgentResult;
    followups: ReadonlyArray<IChatFollowup> | undefined;
    isCanceled: boolean | undefined;
    vote: ChatAgentVoteDirection | undefined;
    voteDownReason?: ChatAgentVoteDownReason;
    usedContext?: IChatUsedContext;
    contentReferences?: ReadonlyArray<IChatContentReference>;
    codeCitations?: ReadonlyArray<IChatCodeCitation>;
    timestamp?: number;
}
export interface IExportableChatData {
    initialLocation: ChatAgentLocation | undefined;
    requests: ISerializableChatRequestData[];
    requesterUsername: string;
    responderUsername: string;
    requesterAvatarIconUri: UriComponents | undefined;
    responderAvatarIconUri: ThemeIcon | UriComponents | undefined;
}
export interface ISerializableChatData1 extends IExportableChatData {
    sessionId: string;
    creationDate: number;
    isImported: boolean;
    isNew?: boolean;
}
export interface ISerializableChatData2 extends ISerializableChatData1 {
    version: 2;
    lastMessageDate: number;
    computedTitle: string | undefined;
}
export interface ISerializableChatData3 extends Omit<ISerializableChatData2, "version" | "computedTitle"> {
    version: 3;
    customTitle: string | undefined;
}
export type ISerializableChatData = ISerializableChatData3;
export type ISerializableChatDataIn = ISerializableChatData1 | ISerializableChatData2 | ISerializableChatData3;
export declare function normalizeSerializableChatData(raw: ISerializableChatDataIn): ISerializableChatData;
export declare function isExportableSessionData(obj: unknown): obj is IExportableChatData;
export declare function isSerializableSessionData(obj: unknown): obj is ISerializableChatData;
export type IChatChangeEvent = IChatInitEvent | IChatAddRequestEvent | IChatChangedRequestEvent | IChatRemoveRequestEvent | IChatAddResponseEvent | IChatSetAgentEvent | IChatMoveEvent | IChatSetHiddenEvent;
export interface IChatAddRequestEvent {
    kind: "addRequest";
    request: IChatRequestModel;
}
export interface IChatChangedRequestEvent {
    kind: "changedRequest";
    request: IChatRequestModel;
}
export interface IChatAddResponseEvent {
    kind: "addResponse";
    response: IChatResponseModel;
}
export declare enum ChatRequestRemovalReason {
    Removal = 0,
    Resend = 1,
    Adoption = 2
}
export interface IChatRemoveRequestEvent {
    kind: "removeRequest";
    requestId: string;
    responseId?: string;
    reason: ChatRequestRemovalReason;
}
export interface IChatSetHiddenEvent {
    kind: "setHidden";
    hiddenRequestIds: Set<string>;
}
export interface IChatMoveEvent {
    kind: "move";
    target: URI;
    range: IRange;
}
export interface IChatSetAgentEvent {
    kind: "setAgent";
    agent: IChatAgentData;
    command?: IChatAgentCommand;
}
export interface IChatInitEvent {
    kind: "initialize";
}
export declare enum ChatModelInitState {
    Created = 0,
    Initializing = 1,
    Initialized = 2
}
export declare class ChatModel extends Disposable implements IChatModel {
    private readonly initialData;
    private readonly _initialLocation;
    private readonly logService;
    private readonly chatAgentService;
    static getDefaultTitle(requests: (ISerializableChatRequestData | IChatRequestModel)[]): string;
    private readonly _onDidDispose;
    readonly onDidDispose: Event<void>;
    private readonly _onDidChange;
    readonly onDidChange: Event<IChatChangeEvent>;
    private _requests;
    private _initState;
    private _isInitializedDeferred;
    private _welcomeMessage;
    get welcomeMessage(): IChatWelcomeMessageContent | undefined;
    private _sampleQuestions;
    get sampleQuestions(): IChatFollowup[] | undefined;
    private _sessionId;
    get sessionId(): string;
    get requestInProgress(): boolean;
    get requestPausibility(): ChatPauseState;
    get hasRequests(): boolean;
    get lastRequest(): ChatRequestModel | undefined;
    private _creationDate;
    get creationDate(): number;
    private _lastMessageDate;
    get lastMessageDate(): number;
    private get _defaultAgent();
    get requesterUsername(): string;
    get responderUsername(): string;
    private readonly _initialRequesterAvatarIconUri;
    get requesterAvatarIconUri(): URI | undefined;
    private readonly _initialResponderAvatarIconUri;
    get responderAvatarIcon(): ThemeIcon | URI | undefined;
    get initState(): ChatModelInitState;
    private _isImported;
    get isImported(): boolean;
    private _customTitle;
    get customTitle(): string | undefined;
    get title(): string;
    get initialLocation(): ChatAgentLocation;
    constructor(initialData: ISerializableChatData | IExportableChatData | undefined, _initialLocation: ChatAgentLocation, logService: ILogService, chatAgentService: IChatAgentService);
    private _deserialize;
    private reviveVariableData;
    private getParsedRequestFromString;
    toggleLastRequestPaused(isPaused?: boolean): void;
    startInitialize(): void;
    deinitialize(): void;
    initialize(welcomeMessage?: IChatWelcomeMessageContent, sampleQuestions?: IChatFollowup[]): void;
    setInitializationError(error: Error): void;
    waitForInitialization(): Promise<void>;
    getRequests(): ChatRequestModel[];
    private _checkpoint;
    get checkpoint(): ChatRequestModel | undefined;
    disableRequests(requestIds: ReadonlyArray<string>): void;
    addRequest(message: IParsedChatRequest, variableData: IChatRequestVariableData, attempt: number, chatAgent?: IChatAgentData, slashCommand?: IChatAgentCommand, confirmation?: string, locationData?: IChatLocationData, attachments?: IChatRequestVariableEntry[], workingSet?: URI[], isCompleteAddedRequest?: boolean): ChatRequestModel;
    setCustomTitle(title: string): void;
    setRequestPaused(request: ChatRequestModel, isPaused: boolean): void;
    updateRequest(request: ChatRequestModel, variableData: IChatRequestVariableData): void;
    adoptRequest(request: ChatRequestModel): void;
    acceptResponseProgress(request: ChatRequestModel, progress: IChatProgress, quiet?: boolean): void;
    removeRequest(id: string, reason?: ChatRequestRemovalReason): void;
    cancelRequest(request: ChatRequestModel): void;
    setResponse(request: ChatRequestModel, result: IChatAgentResult): void;
    completeResponse(request: ChatRequestModel): void;
    setFollowups(request: ChatRequestModel, followups: IChatFollowup[] | undefined): void;
    setResponseModel(request: ChatRequestModel, response: ChatResponseModel): void;
    toExport(): IExportableChatData;
    toJSON(): ISerializableChatData;
    dispose(): void;
}
export declare function updateRanges(variableData: IChatRequestVariableData, diff: number): IChatRequestVariableData;
export declare function canMergeMarkdownStrings(md1: IMarkdownString, md2: IMarkdownString): boolean;
export declare function appendMarkdownString(md1: IMarkdownString, md2: IMarkdownString | string): IMarkdownString;
export declare function getCodeCitationsMessage(citations: ReadonlyArray<IChatCodeCitation>): string;
